#include "MsgHndlr.h"

CMsgHandler::TMap CMsgHandler::InstanceMap;

LRESULT CMsgHandler::GlobalWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	TMap& mp=InstanceMap;
	TWndHandlers* pHndrs=NULL;
	if(mp.find(hWnd)!=mp.end())
		pHndrs=&mp[hWnd];
	else
		return 0;
	TList* pList=&pHndrs->lst; 
	// call PreWndProc functions for all MsgHandlers of this Wnd
	for(TList::reverse_iterator rit=pList->rbegin();rit!=pList->rend();rit++)
	{
		if((*rit)->PreWndProc(hWnd,uMsg,wParam,lParam))
			return 1;
	}
	// call old wndProc
	WNDPROC& wndProc=pHndrs->wndProcOld;
	LRESULT nResBase=0;
	if(wndProc)
		nResBase=::CallWindowProc(wndProc,hWnd,uMsg,wParam,lParam);
	if(uMsg==WM_DESTROY || uMsg==WM_NCDESTROY) // no postprocessing after the window is destroyed
		pList->clear();
	// try to find handlers again ( 'cos handler can be already removed)
	if(mp.find(hWnd)!=mp.end())
		pHndrs=&mp[hWnd];
	else
		return 0;
	// call PostWndProc functions for all MsgHandlers of this Wnd
	for(TList::iterator it=pList->begin();it!=pList->end();it++)
	{
		if((*it)->PostWndProc(hWnd,uMsg,wParam,lParam))
			return nResBase;
	}
	return nResBase;
}

CMsgHandler::CMsgHandler(HWND hWnd)
{
	Initialize(hWnd); 
}

CMsgHandler::~CMsgHandler()
{
	;
}

BOOL CMsgHandler::Initialize(HWND _hWnd)
{
	if(!_hWnd)
		return TRUE;
	if(!::IsWindow(_hWnd))
		return FALSE;
	hWnd=_hWnd;
	TMap::iterator pHndrs=InstanceMap.find(hWnd);
	if(pHndrs==InstanceMap.end())	// if no handlers attached to this Wnd
	{
		TWndHandlers msgHndrs;
		msgHndrs.wndProcOld=(WNDPROC)::SetWindowLong(hWnd,GWL_WNDPROC,(DWORD)stdcallGlobalWndProc);
		msgHndrs.lst.push_back(this);
		InstanceMap[hWnd]=msgHndrs;
	}
	else
		pHndrs->second.lst.push_back(this);
	return TRUE;
}

BOOL CMsgHandler::Terminate()
{
	if(!::IsWindow(hWnd))
		return FALSE;
	TMap::iterator it=InstanceMap.find(hWnd);
	if(it!=InstanceMap.end())
		RemoveFromMap();
	return TRUE;
}

LRESULT CMsgHandler::PreWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return 0;	// it's dummy
}

LRESULT CMsgHandler::PostWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return 0;	// it's dummy
}

BOOL CMsgHandler::RemoveFromMap()
{
	if(!hWnd)
		return TRUE;
	TMap::iterator it=InstanceMap.find(hWnd);
	if(it==InstanceMap.end())
		return TRUE;
	TWndHandlers* pHndrs=&it->second;
	TList& lst=pHndrs->lst;
	for(TList::iterator ls_it=lst.begin();ls_it!=lst.end();ls_it++)
	{
		if(*ls_it==this)
		{
			lst.erase(ls_it);
			if(lst.size()==0)
			{
				// set old wndProc
				if(pHndrs->wndProcOld)
					::SetWindowLong(hWnd,GWL_WNDPROC,(DWORD)pHndrs->wndProcOld);
				// remove from map
				InstanceMap.erase(hWnd);
			}
			break;
		}
	}
	return TRUE;
}

WNDPROC CMsgHandler::GetOldWndProc()
{
	TMap::iterator it=InstanceMap.find(hWnd);
	return it!=InstanceMap.end() ? it->second.wndProcOld : NULL;
}
