///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//// Module contains class which helps to override WNDPROC function
//// of Win32 window object. It calls overridable PreWndProc and 
//// PostWndProc functions before and after previous WndProc function call
////
//// author - Sholomov D.L. (02.07.2001)
//// Cognitive Technologies Ltd. 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _MSG_HNDLR_H
#define _MSG_HNDLR_H

#ifndef _WINDOWS_
#include "windows.h" 
#endif

#pragma warning (disable:4786)

#include <list>
#include <map>
using namespace std;

class CMsgHandler
{
	typedef list<CMsgHandler*> TList;
	typedef struct tagWndHandlers
	{
		TList lst;
		WNDPROC wndProcOld;
		tagWndHandlers() : wndProcOld(NULL) { ; } 
	} TWndHandlers;
	typedef map<HWND,TWndHandlers> TMap;

	static TMap  CMsgHandler::InstanceMap;	// global instance map
	// window procedure for all windows with MsgHandlers attached
	static LRESULT GlobalWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	// stdcall wrapper for GlobalWndProc callback function
	friend LRESULT _stdcall stdcallGlobalWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	BOOL RemoveFromMap();
protected:	
	HWND hWnd;
	WNDPROC GetOldWndProc();
	// Overridables
	virtual LRESULT PreWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	virtual LRESULT PostWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
public:
	// Construction
	CMsgHandler(HWND hWnd=NULL); 
	virtual ~CMsgHandler();
	// Initialization
	BOOL Initialize(HWND hWnd=NULL); // also attaches MsgHandler to the window
	BOOL Terminate();	// also detaches MsgHandler from the window
};

inline LRESULT CALLBACK stdcallGlobalWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return CMsgHandler::GlobalWndProc(hWnd,uMsg,wParam,lParam);
}

#endif	// _MSG_HNDLR_H
